/*
 * Copyright (c) 2006-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 118 $
 * $LastChangedDate: 2007-02-20 17:26:56 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.fm.templating;

import java.io.Writer;
import net.sf.json.JSONObject;
import freemarker.template.*;
import com.codedroids.templating.TemplatingException;
import com.codedroids.oc.freemarker.CmsHash;
import com.codedroids.oc.freemarker.VFSTemplateLoader;
import com.codedroids.oc.templating.*;
import com.codedroids.oc.*;
import java.io.IOException;
import java.util.Map;

public class FMTemplateEngine extends A_OCTemplateEngine 
{
	String templateName;
	Template template;
	
	public FMTemplateEngine() {
		super();
	}
	
	public void setConfiguration(JSONObject configuration) {
		super.setConfiguration(configuration);
		templateName = (String)configuration.get(OCTemplateEngineFactory.TEF_CONFIG_TEMPLATE);
	}
	
	/**
	 * Initialises the instance with the given configuration. I.e. do not call it
	 * before the configuration is set.
	 * <p>Default implementation do nothing other than complain if no config is present
	 * @return
	 */
	public void init() {
		Configuration cfg = new Configuration();
		String templFolder = PathUtils.folderFromPath(templateName);
		if(templFolder == null)
			templFolder = getCms().info("opencms.request.folder");
		cfg.setTemplateLoader(new VFSTemplateLoader(getCms().getCmsObject(), templFolder));
		cfg.setLocalizedLookup(true); 
		cfg.setTemplateExceptionHandler(TemplateExceptionHandler.IGNORE_HANDLER);
		String templFile = PathUtils.filenameFromPath(templateName);
		if(templFile==null) {
			templFile = PathUtils.filenameFromPath(getCms().info("opencms.request.folder"));
			templFile = PathUtils.stripExtension(templFile, ".") + ".ftl";
		}
		String encoding = getConfiguration().optString("encoding", "UTF-8");
		try {
			template = cfg.getTemplate(templFile, encoding);
		} 
		catch(IOException e) {
			template = null;
		}
	}	
	

	/**
	 * The context is assumed to be a Map containing the context. Note that this method automatically
	 * adds the CmsHash context.
	 * 
	 * @see com.codedroids.templating.A_TemplateEngine#process(java.lang.Object, java.io.Writer)
	 */
	public void process(Map context, Writer writer) throws TemplatingException {
		CmsHash root = new CmsHash(getCms());
		if(context != null) {
			root.putAll(context);
		}
        try {
        		template.process(context, writer);
        } catch(Exception e) {
        		throw new TemplatingException("Error processing the \"" + templateName + "\" template: ", e);
        }
	}
}
